#ifndef THUMBNAILMANAGER_H
#define THUMBNAILMANAGER_H

#include <QObject>
#include <KWindowSystem>
#include <QPixmap>
#include <QMap>
#include <QMutex>
#include <QSettings>
#include <QFileSystemWatcher>
#include <QDateTime>
#include <QDir>

class ThumbnailManager : public QObject
{
    Q_OBJECT
private:
    explicit ThumbnailManager(QObject *parent = nullptr);

public:
    static ThumbnailManager * getIns();

    QPixmap getThumbnail(WId id);

    QPixmap getWallPaper();
    QPixmap getWallPaper(int desktop);

    //获取模糊处理之后的桌面壁纸
    QPixmap getAdjustWallPaper();

    void refreshScreenShot(int deskNum);

Q_SIGNALS:
    void windowAdded(WId id);
    void windowRemoved(WId id);
    void windowChanged(WId window, NET::Properties prop, NET::Properties2 prop2);
public Q_SLOTS:
    void onKwinWindowAdd(WId id);
    void onKwinWindowRemove(WId id);
    void onKwinWindowChanged(WId id, NET::Properties p1, NET::Properties2 p2);
    void onDesktopChanged(int desktop);
    void onDesktopDelete(int desktop);

private Q_SLOTS:
    void onBackgroundChanged();

private:
    void removeThumbnail(WId id);
    void updateThumbnail(WId id);
    QString getBackgroundSettingPath();

private:
    QFileSystemWatcher mWatcher;
    QMutex m_thumbnailsLock;
    QMap<WId, QPixmap> m_thumbnails;
    QMap<int, QPixmap> m_screenShot;

    QString m_wallPaperPath;
    QPixmap m_wallPaper;
    QPixmap m_adjustWallPaper;
    QDateTime mCurTime;
    QMap<WId, QDateTime> m_freReduc;
};
#endif // THUMBNAILMANAGER_H
