#include "notifypopup.h"

#include <QHBoxLayout>
#include <QEvent>
#include <KF5/KWindowSystem/KWindowEffects>
#include <QApplication>
#include <QDesktopWidget>
#include <QDBusConnection>
#include <QDBusInterface>
#include <QDebug>

NotifyPopup::NotifyPopup(INdePanelPlugin *plugin, QWidget *parent) :
    QWidget(parent),
    mPlugin(plugin),
    mCompositorOpen(false)
{
    this->setObjectName("NotifyPopup");
    this->setWindowFlags(Qt::FramelessWindowHint |
                         Qt::WindowStaysOnTopHint |
                         Qt::Tool |
                         Qt::X11BypassWindowManagerHint);
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setAttribute(Qt::WA_NoSystemBackground, false);
    KWindowEffects::enableBlurBehind(this->winId(), true);

    //icon layout
    m_labelIcon = new QLabel(this);
    m_labelIcon->setObjectName("label_icon");
    m_labelIcon->setFixedSize(NOTIFY_ICON_SIZE);

    m_layout_icon = new QVBoxLayout;
    m_layout_icon->setMargin(0);
    m_layout_icon->setContentsMargins(0,0,0,0);
    m_layout_icon->setSpacing(0);
    m_layout_icon->addWidget(m_labelIcon);
    m_layout_icon->addStretch();

    //actions btns layout
    m_layout_actions = new QGridLayout;
    m_layout_actions->setContentsMargins(0,10,0,10);
    m_layout_actions->setSpacing(5);
    m_frame_actions = new QFrame(this);
    m_frame_actions->setLayout(m_layout_actions);

    //text layout
    m_labelApplication = new QLabel(this);
    m_labelApplication->setObjectName("label_application");
    QFont f = m_labelApplication->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelApplication->setFont(f);
    m_labelApplication->setTextFormat(Qt::PlainText);
    m_labelApplication->setMinimumWidth(200);

    m_labelSummary = new QLabel(this);
    m_labelSummary->setObjectName("label_summary");
    f = m_labelSummary->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelSummary->setFont(f);
    m_labelSummary->setTextFormat(Qt::RichText);
    m_labelSummary->setWordWrap(true);
    m_labelSummary->setMinimumWidth(200);

    m_labelBody = new QLabel(this);
    m_labelBody->setObjectName("label_body");
    f = m_labelBody->font();
    f.setBold(true);
    f.setPixelSize(14);
    m_labelBody->setFont(f);
    m_labelBody->setTextFormat(Qt::RichText);
    m_labelBody->setWordWrap(true);
    m_labelBody->setOpenExternalLinks(true);
    m_labelBody->setMinimumWidth(200);

    m_layout_text = new QVBoxLayout;
    m_layout_text->setMargin(0);
    m_layout_text->setSpacing(5);
    m_layout_text->setContentsMargins(0,0,0,0);
    m_layout_text->addWidget(m_labelApplication);
    m_layout_text->addWidget(m_labelSummary);
    m_layout_text->addWidget(m_labelBody);
    m_layout_text->addWidget(m_frame_actions);
    m_layout_text->addStretch(0);

    //btn layout
    m_btn_close = new QToolButton(this);
    m_btn_close->setObjectName("btn_close");
    m_btn_close->setFixedSize(28,28);
    m_btn_close->setIconSize(QSize(28, 28));
    connect(m_btn_close, &QToolButton::clicked, this, &NotifyPopup::slotClose);

    m_layout_btn = new QVBoxLayout;
    m_layout_btn->setMargin(0);
    m_layout_btn->setSpacing(0);
    m_layout_btn->setContentsMargins(0,0,0,0);
    m_layout_btn->addWidget(m_btn_close);
    m_layout_btn->addStretch(0);

    mBackground = new QWidget(this);
    mBackground->setObjectName("NotifyPopupBackgroundWidget");
    //main layout
    m_layout = new QHBoxLayout;
    m_layout->setContentsMargins(12,16,12,16);
    m_layout->setSpacing(10);
    m_layout->addLayout(m_layout_icon);
    m_layout->addLayout(m_layout_text);
    m_layout->addLayout(m_layout_btn);
    m_layout->addStretch(0);
    mBackground->setLayout(m_layout);

    QHBoxLayout * layout = new QHBoxLayout;
    layout->setContentsMargins(0,0,0,0);
    layout->setSpacing(0);
    layout->addWidget(mBackground);
    this->setLayout(layout);

    mCompositorOpen = mPlugin->panel()->compositorOpen();
    mBackground->setProperty("compositor", mCompositorOpen);
}

NotifyPopup::~NotifyPopup()
{

}

void NotifyPopup::setValues(NotifyMessage nm)
{
    m_notifyMsg = nm;

    bool isScreensaver = false;
    QDBusMessage message = QDBusMessage::createMethodCall(QString::fromLatin1("org.nde.ScreenSaver"),
                                                          QString::fromLatin1("/ScreenSaver"),
                                                          QString::fromLatin1("org.nde.ScreenSaver"),
                                                          QString::fromLatin1("getLockState"));
    QDBusMessage reply = QDBusConnection::sessionBus().call(message);
    if(reply.arguments().count() > 0)
    {
        isScreensaver = reply.arguments().first().toBool();
    }

    if(isScreensaver) return;


    this->layout()->setEnabled(false);

    m_labelIcon->setPixmap(nm.m_icon);

    m_labelApplication->setText(nm.m_name);
    //简单描述
    m_labelSummary->setText(nm.m_summary);

    //详细描述
    QString formatted(nm.m_body);
    m_labelBody->setText(formatted.replace(QL1C('\n'), QStringLiteral("<br/>")));

    //动作
    //解析actions中的动作(actions<<acion<<desc，删除动作的描述
    while(m_btn_actions.count())
    {
        QToolButton * btn = m_btn_actions.takeFirst();
        m_layout_actions->removeWidget(btn);
        delete  btn;
    }
    int row = 0;
    int col = 0;
    for (int i = 0; i < nm.m_actions.count(); i += 2)
    {
        if(col > 1)
        {
            col = 0;
            row += 1;
        }

        QString key = nm.m_actions[i].first;
        QString desc = nm.m_actions[i].second;
        QToolButton * btn = new QToolButton(this);
        btn->setFixedSize(QSize(100, 24));
        connect(btn, &QToolButton::clicked, this, &NotifyPopup::slotActionBtnClicked);
        btn->setText(desc);
        btn->setProperty("action_key", key);
        m_layout_actions->addWidget(btn, row, col);

        col++;

        m_btn_actions.append(btn);
    }
    m_frame_actions->setVisible(m_btn_actions.count());


    this->layout()->setEnabled(true);

    mPlugin->panel()->willShowWindow(this);
    this->show();
    this->adjustSize();
//    this->activateWindow();

    //计算位置
    QRect rect = qApp->desktop()->screenGeometry();
    int pos_x,pos_y;
    switch (mPlugin->panel()->position())
    {
    case INdePanel::PositionTop:
        rect.adjust(0, mPlugin->panel()->globalGeometry().height(),0,0);
        pos_x = rect.width()-this->width();
        pos_y = mPlugin->panel()->globalGeometry().height()+4;
        break;
    case INdePanel::PositionBottom:
        rect.adjust(0,0,0,-mPlugin->panel()->globalGeometry().height());
        pos_x = rect.width()-this->width();
        pos_y = rect.height()-this->height()-4;
        break;

    case INdePanel::PositionLeft:
        rect.adjust(mPlugin->panel()->globalGeometry().width(),0,0,0);
        pos_x = mPlugin->panel()->globalGeometry().width()+4;
        pos_y = rect.height()-this->height();
        break;

    case INdePanel::PositionRight:
        rect.adjust(0,0,-mPlugin->panel()->globalGeometry().width(),0);
        pos_x = rect.width()-this->width()-4;
        pos_y = rect.height()-this->height();
        break;
    }
    this->move(pos_x,pos_y);
}

void NotifyPopup::realign()
{
    if(mCompositorOpen != mPlugin->panel()->compositorOpen())
    {
        mCompositorOpen = mPlugin->panel()->compositorOpen();
        mBackground->setProperty("compositor", mCompositorOpen);
    }
}

bool NotifyPopup::event(QEvent *event)
{
    if(event->type() == QEvent::ActivationChange)
    {
        if(!this->isActiveWindow())
        {
            this->hide();
        }
    }
    return QWidget::event(event);
}

void NotifyPopup::slotActionBtnClicked()
{
    QToolButton *btn = qobject_cast<QToolButton *>(sender());
    if(btn)
    {
        QString key = btn->property("action_key").toString();
        if(!key.isEmpty()) Q_EMIT actionTriggered(m_notifyMsg.m_id, key);
    }

    this->hide();
}

void NotifyPopup::slotClose()
{
    this->hide();
}

