#include "ndethumbnailbutton.h"
#include "ndetaskbar.h"
#include <QLabel>
#include <QDebug>
#include <QStyleOption>
#include <QPainter>
#include <QMouseEvent>
#include <KWindowSystem/NETWM>
#include <QX11Info>
#include <QApplication>
#include <QDesktopWidget>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#define TITLE_HEIGHT 22

NdeThumbnailButton::NdeThumbnailButton(const WId window, NdeTaskBar * taskBar, QWidget *parent) :
    QFrame(parent),
    mWindow(window),
    mParentTaskBar(taskBar)
{
    setObjectName("thumbnailButton");
    QVBoxLayout *layout = new QVBoxLayout;
    closeButton = new QToolButton(this);
    closeButton->setObjectName("closeButton");
    closeButton->setFixedHeight(TITLE_HEIGHT);
    connect(closeButton, &QToolButton::clicked, this, &NdeThumbnailButton::closeApplication);
    windowTitle = new QLabel;
    windowTitle->setObjectName("windowTitle");
    thumbnailPixmap = new QLabel;
    thumbnailPixmap->setScaledContents(true);
    windowTitle->setFixedHeight(TITLE_HEIGHT);


    closeButton->hide();
    QHBoxLayout *titleLayout = new QHBoxLayout;
    QHBoxLayout *controlLayout = new QHBoxLayout;
    titleLayout->addWidget(windowTitle, 1, Qt::AlignLeft | Qt::AlignVCenter);
    titleLayout->addWidget(closeButton, 0, Qt::AlignRight | Qt::AlignVCenter);
    titleLayout->setContentsMargins(0, 0, 0, 0);

    layout->addItem(titleLayout);
    layout->addWidget(thumbnailPixmap, 1, Qt::AlignHCenter | Qt::AlignVCenter);
    layout->addItem(controlLayout);

    setLayout(layout);

    updateTitle();
}
NdeThumbnailButton::~NdeThumbnailButton()
{

}
bool NdeThumbnailButton::isApplicationActive() const
{
    return KWindowSystem::activeWindow() == mWindow;
}
bool NdeThumbnailButton::isOnDesktop(int desktop) const
{
    return KWindowInfo(mWindow, NET::WMDesktop).isOnDesktop(desktop);
}
void NdeThumbnailButton::closeApplication()
{
    NETRootInfo(QX11Info::connection(), NET::CloseWindow).closeWindowRequest(mWindow);
}
void NdeThumbnailButton::setUrgencyHint(bool set)
{
    if (mUrgencyHint == set)
        return;

    if (!set)
        KWindowSystem::demandAttention(mWindow, false);

    mUrgencyHint = set;
    setProperty("urgent", set);
    style()->unpolish(this);
    style()->polish(this);
    update();
}
void NdeThumbnailButton::raiseApplication()
{
    KWindowInfo info(mWindow, NET::WMDesktop | NET::WMState | NET::XAWMState);
    if (parentTaskBar()->raiseOnCurrentDesktop() && info.isMinimized())
    {
        KWindowSystem::setOnDesktop(mWindow, KWindowSystem::currentDesktop());
    }
    else
    {
        int winDesktop = info.desktop();
        if (KWindowSystem::currentDesktop() != winDesktop)
            KWindowSystem::setCurrentDesktop(winDesktop);
    }
    KWindowSystem::activateWindow(mWindow);

    setUrgencyHint(false);
}
bool NdeThumbnailButton::isOnCurrentScreen() const
{
    return QApplication::desktop()->screenGeometry(parentTaskBar()).intersects(KWindowInfo(mWindow, NET::WMFrameExtents).frameGeometry());
}
void NdeThumbnailButton::updateTitle()
{
    KWindowInfo info(mWindow, NET::WMVisibleName | NET::WMName);
    QString title = info.visibleName().isEmpty() ? info.name() : info.visibleName();
    windowTitle->setText(title.replace("&", "&&"));
    setToolTip(title);
}

QString NdeThumbnailButton::title() const
{
    return windowTitle->text();
}
void NdeThumbnailButton::addThumbnail(QPixmap pixmap)
{
    thumbnailPixmap->setPixmap(pixmap);
}
void NdeThumbnailButton::enterEvent(QEvent *event)
{
    closeButton->show();
    QFrame::enterEvent(event);
}

void NdeThumbnailButton::leaveEvent(QEvent *event)
{
    closeButton->hide();
    QFrame::enterEvent(event);
}
void NdeThumbnailButton::mousePressEvent(QMouseEvent* event)
{
    const Qt::MouseButton b = event->button();

    if (Qt::MidButton == b && parentTaskBar()->closeOnMiddleClick())
        closeApplication();
    QFrame::mousePressEvent(event);
}
void NdeThumbnailButton::mouseReleaseEvent(QMouseEvent* event)
{
    //点击了关闭按钮不要触发click事件
    if(closeButton && closeButton->isVisible() && closeButton->geometry().contains(event->pos()))
    {
        //do nothing
    }
    else
    {
        if (event->button() == Qt::LeftButton)
        {
            raiseApplication();
        }
        Q_EMIT clicked();
    }
    QFrame::mouseReleaseEvent(event);
}
void NdeThumbnailButton::minimizeApplication()
{
    KWindowSystem::minimizeWindow(mWindow);
}
void NdeThumbnailButton::setAutoRotation(bool value, INdePanel::Position position)
{
    if (value)
    {
        switch (position)
        {
        case INdePanel::PositionTop:
        case INdePanel::PositionBottom:
            setOrigin(Qt::TopLeftCorner);
            break;

        case INdePanel::PositionLeft:
            setOrigin(Qt::BottomLeftCorner);
            break;

        case INdePanel::PositionRight:
            setOrigin(Qt::TopRightCorner);
            break;
        }
    }
    else
        setOrigin(Qt::TopLeftCorner);
}
void NdeThumbnailButton::setOrigin(Qt::Corner newOrigin)
{
    if (mOrigin != newOrigin)
    {
        mOrigin = newOrigin;
        update();
    }
}
